#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_types.h"
#include "driverlib/gpio.h"
#include "driverlib/interrupt.h"
#include "driverlib/sysctl.h"
#include "driverlib/uart.h"

#include "libraries/staticCircularBuffer.h"

#include "UartDriverSimple.h"

static void UART_DRIVER_SIMPLE_readUartData(void );
void UART_SIMPLE_DRIVER_fromBufferToUART(void);

//BUFFER DE RECEPCION
STATIC_CIRCULAR_BUFFER inBuffer;
//BUFFER DE EMISION
STATIC_CIRCULAR_BUFFER outBuffer;

int UART_DRIVER_SIMPLE_open(){
	//
    // Enable the peripherals used by this example.
    //
    //SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
   // SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    //
    // Set GPIO A0 and A1 as UART pins.
    //
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
    UARTDisable(0);
    //
    // Configure the UART for 115,200, 8-N-1 operation.
    //
    UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(), UART_BAUD_RATE,
                            (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                             UART_CONFIG_PAR_NONE));

    UARTEnable(0);
    UARTFIFOLevelSet( 0, UART_FIFO_TX1_8, UART_FIFO_RX1_8);
    //
    // Enable the UART interrupt.
    //
    IntEnable(INT_UART0);
    UARTIntEnable(UART0_BASE, UART_INT_TX | UART_INT_RX | UART_INT_RT);
    //
    STATIC_CIRCULAR_BUFFER_init(&inBuffer);
    STATIC_CIRCULAR_BUFFER_init(&outBuffer);
}

void UART_DRIVER_SIMPLE_close(){
	UARTDisable(UART0_BASE);
	IntDisable(INT_UART0);
}

//*****************************************************************************
//
// Send a string to the UART.
//
//*****************************************************************************
//int UART_DRIVER_SIMPLE_send(int nPort, unsigned char *p, int *pSize){
int UART_DRIVER_SIMPLE_send(const unsigned char *pucBuffer, unsigned long ulCount)
{
		char data;
		STATIC_CIRCULAR_BUFFER_insertChars( &outBuffer , pucBuffer, ulCount);
		//
	    // Loop while there are more characters to send.
	    //
		UART_SIMPLE_DRIVER_fromBufferToUART();
		return 0;
}

int UART_DRIVER_SIMPLE_recv(  unsigned char *p){
	int length = 0;
	int i;
	length = STATIC_CIRCULAR_BUFFER_dataCount(&inBuffer);
	for ( i = 0 ; i < length ; i++ ){
		p[i] = STATIC_CIRCULAR_BUFFER_readChar(&inBuffer );
	}

	return i;
}

int UART_DRIVER_SIMPLE_recvCount(  unsigned char *p, unsigned long *pSize){
	int length,i;
	length = STATIC_CIRCULAR_BUFFER_dataCount(&inBuffer);
	if (length < *pSize) *pSize = length;
	for ( i = 0 ; i < *pSize ; i++ ){
		p[i] = STATIC_CIRCULAR_BUFFER_readChar(&inBuffer );
	}

	return 0;
}

//*****************************************************************************
//
// The UART interrupt handler.
//
//*****************************************************************************
void UART0_IRQHandler(void)
{
	//0-Leer el estado de la interrupcion y borrar la interrupcion UART
	//1-Diferenciar si la interrupcion es TX o RX
	//2-Si es una interrupcion de RX
	//	a)Lee todos los caracterteres de forma no bloqueante mientras esten disponibles en la UART
	//	b)Segun los va leyendo los inserta en el buffer.
	//3-Si es una interrupcion de TX
    unsigned long ulStatus;
	//
    // Get the interrrupt status.
    //
    ulStatus = UARTIntStatus(UART0_BASE, true);
    UARTIntClear(UART0_BASE, ulStatus);
    if(ulStatus & UART_INT_TX)
     {
      // nInts++;
       if( STATIC_CIRCULAR_BUFFER_dataCount(&outBuffer) > 0  ){
    	   /*if(!uarts[0].writingToBuf) {toFIFO(0);}
    	   else uarts[0].outIntPending=0;*/
    	   UART_SIMPLE_DRIVER_fromBufferToUART();
    	   UARTIntClear(UART0_BASE, UART_INT_TX);
       }

     }


    if(ulStatus & UART_INT_RX | UART_INT_RT){
       UART_DRIVER_SIMPLE_readUartData();
       UARTIntClear(UART0_BASE, UART_INT_RX);
    }
}


void UART_DRIVER_SIMPLE_readUartData(){
	 int freeDataCount;
	 unsigned int data;
	 char str[30], str1[30];
	 int i=0;
	 //No vaciamos todo el buffer del device Uart
	 freeDataCount = STATIC_CIRCULAR_BUFFER_freeDataCount(&inBuffer);
	 //Solo los huecos disponibles.
	 while( UARTCharsAvail(UART0_BASE) && ( freeDataCount-- >0 ) ){
	    data=UARTCharGetNonBlocking(UART0_BASE);
	    //antes de enviaxr hay que filtrar
	    //el echo
	    UART_DRIVER_SIMPLE_send(&data, 1);
	    STATIC_CIRCULAR_BUFFER_insertChar( &inBuffer , (unsigned char)data );
	    //i++;
	 }
	/* str1[i]='\0';
	 for (i = 0; i < STATIC_CIRCULAR_BUFFER_dataCount(&inBuffer); i++ ){
		 str[i] = STATIC_CIRCULAR_BUFFER_readChar(&inBuffer);
	 }
	 str[i]=':';
	 str[i+1]=i;
	 eraseConsoleLine(12);
	 consolePrintStr(2,12, str);
	 refreshConsoleLine(12);*/
	 //eraseConsoleLine(6);
	 //consolePrintStr(2,6, str1);
	 //refreshConsoleLine(6);
}

void UART_SIMPLE_DRIVER_fromBufferToUART(){
	int n = 0;
	char data;
	n = STATIC_CIRCULAR_BUFFER_dataCount(&outBuffer);
	while( ( n-- > 0 ) 	&& ( UARTSpaceAvail(UART0_BASE)  ) ){
					data = STATIC_CIRCULAR_BUFFER_readChar( &outBuffer);
					UARTCharPutNonBlocking(UART0_BASE, data);
	}
}

int UART_DRIVER_SIMPLE_peekChar(  char c ){
	int ret = 0;
	ret = STATIC_CIRCULAR_BUFFER_existChar( &inBuffer, c);
	return ret;
}
