#include "keypad.h"


//*****************************************************************************
//
// The debounced state of the five push buttons.  The bit positions correspond
// to:
//
//     0 - Up
//     1 - Down
//     2 - Left
//     3 - Right
//     4 - Select
//
//*****************************************************************************
unsigned char keypad_switches = 0x1f;
unsigned char changed_data = 0x00;
unsigned char bouncingCounter = 0x00;
unsigned char debounced_data = 0x1f;
tBoolean bouncing = false;

void KEYPAD_init(){
	//
	// Configure the GPIOs used to read the state of the on-board push buttons.
	//
	GPIOPinTypeGPIOInput( GPIO_PORTE_BASE , GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3);
	GPIOPadConfigSet( GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3, GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD_WPU);
	GPIOPinTypeGPIOInput( GPIO_PORTF_BASE, GPIO_PIN_1);
	GPIOPadConfigSet( GPIO_PORTF_BASE, GPIO_PIN_1 , GPIO_STRENGTH_2MA , GPIO_PIN_TYPE_STD_WPU);
}

void KEYPAD_readEventWithBouncing(){
	unsigned long pressed_data;

	pressed_data = (
	    					GPIOPinRead( GPIO_PORTE_BASE , (GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3) )
	    					|
	    					(GPIOPinRead(GPIO_PORTF_BASE, GPIO_PIN_1) << 3)
	                   );

    changed_data = keypad_switches ^ pressed_data;
    keypad_switches = pressed_data;
    keypad_switches = keypad_switches & 0x1f;
    //SIN PONER NADA .TAMBIEN FUNCIONA
}

/*
 * Esto funciona si la tarea de lectura es periodica.
 *
 */
void KEYPAD_readEvent(){
	unsigned long pressed_data;
	tBoolean isAnyPressed = false;
	//Para validar el funcionamiento hay que depurar la aplicacion
	do{
		pressed_data = (
		    					GPIOPinRead( GPIO_PORTE_BASE , (GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3) )
		    					|
		    					(GPIOPinRead(GPIO_PORTF_BASE, GPIO_PIN_1) << 3)
		                   );

		 if  (   ( pressed_data == keypad_switches  ) && ( bouncing==true) ) {
		    	//hay algo apretado
		    	bouncingCounter++;
		 }
		 else{
			 	//no se ha apretado
		        //se ha cambiado de estado. comienzo de la cuenta cuando se apriete algo.
		    	bouncingCounter = 0;
		    	bouncing = true;
		    	changed_data = keypad_switches ^ pressed_data;
		 }

		 if ( bouncingCounter == 10 ){
		    	bouncing = false;
		    	changed_data = debounced_data ^ pressed_data;
		    	debounced_data = pressed_data & 0x1f;

		 }
		 keypad_switches = pressed_data;
	}while(bouncing == true);

}

void KEYPAD_readEventWithDelay(){
	unsigned long pressed_data;
	tBoolean isAnyPressed = false;
	int delay = 0;
	bouncing = true;
	pressed_data = (
   					GPIOPinRead( GPIO_PORTE_BASE , (GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3) )
   					|
   					(GPIOPinRead(GPIO_PORTF_BASE, GPIO_PIN_1) << 3)
                   );

	if  (   ( pressed_data != keypad_switches  ) )  {
		    	keypad_switches = pressed_data;
				for (delay = 0; delay < 120; delay++ );
		    	pressed_data = (
		    					GPIOPinRead( GPIO_PORTE_BASE , (GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3) )
		    					|
		    					(GPIOPinRead(GPIO_PORTF_BASE, GPIO_PIN_1) << 3)
   			                   );
		    	if (pressed_data == keypad_switches){
		    		bouncing = false;
    		    	changed_data = debounced_data ^ pressed_data;
    		    	debounced_data = pressed_data & 0x1f;
		    	}
	}
	keypad_switches = pressed_data;

}

unsigned char KEYPAD_readData(){
	return keypad_switches;
}

tBoolean  KEYPAD_isPressed(unsigned char key){
	tBoolean ret = false;
	if (keypad_switches  == key){
		ret = true;
	}
	return ret;
}

tBoolean  KEYPAD_isPressedDown(){
	tBoolean ret = false;
	unsigned char changedBit;
	changedBit = changed_data  & 0x02;
	if ( ( changedBit != 0)  && (bouncing == false )){
		if ( debounced_data  == KEY_DOWN){
				ret = true;
		}
	}
	return ret;
}

tBoolean  KEYPAD_isPressedUp(){
	tBoolean ret = false;
	unsigned char changedBit;
	changedBit = changed_data  & 0x01;
		if ( ( changedBit != 0)  && (bouncing == false )){
			if ( debounced_data  == KEY_UP){
				ret = true;
			}
		}
	return ret;
}

tBoolean  KEYPAD_isPressedLeft(){
	tBoolean ret = false;
	unsigned char changedBit;
	changedBit = changed_data  & 0x04;
	if ( ( changedBit != 0)  && (bouncing == false )){
		if (debounced_data  == KEY_LEFT){
			ret = true;
		}
	}
	return ret;
}

tBoolean  KEYPAD_isPressedRight(){
	tBoolean ret = false;
	unsigned char changedBit;
	changedBit = changed_data  & 0x08;
	if( ( changedBit != 0)  && (bouncing == false )){
		if (debounced_data  == KEY_RIGHT){
			ret = true;
		}
	}
	return ret;
}


tBoolean  KEYPAD_isPressedSelect(){
	tBoolean ret = false;
	unsigned char changedBit;
	changedBit = changed_data  & 0x10;
	if ( (changedBit != 0) && (bouncing == false ) ){
			if ( debounced_data  == KEY_SELECT){
				ret = true;
			}
	}
	return ret;
}






